#ifndef _MY_BITREV_H
#define _MY_BITREV_H

#include "types.h"

/**
 * Here is a generalised bit reversal program
 * @x: word to get bits reversed
 * @k: key, explained below
 * for k = 31, it reverses the bits of word(32 bit)
 * for k = 24, it reverses the bytes in word
 * for k = 7, it reverses the bits in every byte without
 * changing the positions of bytes in a word
 * and for k = 16 it swaps the left and right halves of a
 * word
 */

static inline u32 gen_bit_rev(u32 x, u32 k)
{
       if(k & 1)
               x = (x & 0x55555555) << 1 | (x & 0xaaaaaaaa) >> 1;
       if(k & 2)
               x = (x & 0x33333333) << 2 | (x & 0xcccccccc) >> 2;
       if(k & 4)
               x = (x & 0x0f0f0f0f) << 4 | (x & 0xf0f0f0f0) >> 4;
       if(k & 8)
               x = (x & 0x00ff00ff) << 8 | (x & 0xff00ff00) >> 8;
       if(k & 16)
               x = (x & 0x0000ffff) << 16 | (x & 0xffff0000) >> 16;

       return x;
}

/**
 * bitrev8(): function for reversing bits of a byte
 * @byte: the byte to get its bits reversed
 */

static inline u8 bitrev8(u8 byte)
{
      u8 k = 7;
      if(k & 1)
              byte = (byte & 0x5555) << 1 | (byte & 0xaaaa) >> 1;
      if(k & 2)
              byte = (byte & 0x3333) << 2 | (byte & 0xcccc) >> 2;
      if(k & 4)
              byte = (byte & 0x0f0f) << 4 | (byte & 0xf0f0) >> 4;
      return byte;
}

extern u32 bitrev32(u32 in);

#endif /* _MY_BITREV_H */
